import logging
import sys

import structlog


def configure_logging(level: int = logging.INFO, file_name: str = None) -> None:
    """
    Configures logging for both the standard library logging and structlog.

    This function sets up a logging configuration to enable consistent and
    structured logging across an application. It configures the root logger from
    the standard library as well as structlog with appropriate handlers,
    formatters, and processors.

    Args:
        level (int): The logging level to be used (default is logging.INFO).
        file_name (str, optional): The name of the file to log messages to. If not
            provided, logging will be restricted to the console.

    Returns:
        None
    """
    # 1) Configure the standard library root logger
    #    All handlers you add will inherit this level.
    logging.basicConfig(
        format="%(message)s",
        stream=sys.stdout,
        level=level,
    )
    root_logger = logging.getLogger()
    root_logger.handlers.clear()
    # Create formatter
    formatter = logging.Formatter('[%(asctime)s] %(levelname)s in %(name)s: %(message)s')

    # Create standard output handler
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setLevel(level)
    console_handler.setFormatter(formatter)

    root_logger.addHandler(console_handler)

    if file_name:
        file_handler = logging.FileHandler(file_name)
        file_handler.setLevel(level)
        file_handler.setFormatter(formatter)
        root_logger.addHandler(file_handler)

    # Set the overall logging level for the root logger
    root_logger.setLevel(level)

    # 2) Tell structlog to filter by level first,
    #    then add any other processors you need.
    structlog.configure(
        processors=[
            # If log level is too low, abort pipeline and throw away log entry.
            structlog.stdlib.filter_by_level,
            # Add the name of the logger to event dict.
            structlog.stdlib.add_logger_name,
            # Add log level to event dict.
            structlog.stdlib.add_log_level,
            # Perform %-style formatting.
            structlog.stdlib.PositionalArgumentsFormatter(),
            # Add a timestamp in ISO 8601 format.
            structlog.processors.TimeStamper(fmt="iso"),
            # If the "stack_info" key in the event dict is true, remove it and
            # render the current stack trace in the "stack" key.
            structlog.processors.StackInfoRenderer(),
            # If the "exc_info" key in the event dict is either true or a
            # sys.exc_info() tuple, remove "exc_info" and render the exception
            # with traceback into the "exception" key.
            structlog.processors.format_exc_info,
            # If some value is in bytes, decode it to a Unicode str.
            structlog.processors.UnicodeDecoder(),
            # Add callsite parameters.
            structlog.processors.CallsiteParameterAdder(
                {
                    structlog.processors.CallsiteParameter.FILENAME,
                    structlog.processors.CallsiteParameter.FUNC_NAME,
                    structlog.processors.CallsiteParameter.LINENO,
                }
            ),
            # Render the final event dict as JSON.
            structlog.processors.JSONRenderer()
        ],
        # `wrapper_class` is the bound logger that you get back from
        # get_logger(). This one imitates the API of `logging.Logger`.
        wrapper_class=structlog.stdlib.BoundLogger,
        # `logger_factory` is used to create wrapped loggers that are used for
        # OUTPUT. This one returns a `logging.Logger`. The final value (a JSON
        # string) from the final processor (`JSONRenderer`) will be passed to
        # the method of the same name as that you've called on the bound logger.
        logger_factory=structlog.stdlib.LoggerFactory(),
        # logger_factory=structlog.WriteLoggerFactory(
        #     file=Path(file_name).open("wt")
        # ),
        # Effectively freeze configuration after creating the first bound
        # logger.
        cache_logger_on_first_use=True,
    )



import datetime
from pathlib import Path

from ziplime.utils.calendar_utils import get_calendar

from ziplime.assets.domain.ordered_contracts import CHAIN_PREDICATES
from ziplime.assets.repositories.sqlalchemy_adjustments_repository import SqlAlchemyAdjustmentRepository
from ziplime.assets.repositories.sqlalchemy_asset_repository import SqlAlchemyAssetRepository
from ziplime.assets.services.asset_service import AssetService
from ziplime.core.algorithm_file import AlgorithmFile
from ziplime.data.services.data_source import DataSource
from ziplime.finance.commission import PerShare, DEFAULT_PER_SHARE_COST, DEFAULT_MINIMUM_COST_PER_EQUITY_TRADE, \
    PerContract, DEFAULT_PER_CONTRACT_COST, DEFAULT_MINIMUM_COST_PER_FUTURE_TRADE, EquityCommissionModel, \
    FutureCommissionModel
from ziplime.finance.constants import FUTURE_EXCHANGE_FEES_BY_SYMBOL
from ziplime.finance.metrics import default_metrics
from ziplime.finance.slippage.fixed_basis_points_slippage import FixedBasisPointsSlippage
from ziplime.finance.slippage.slippage_model import DEFAULT_FUTURE_VOLUME_SLIPPAGE_BAR_LIMIT
from ziplime.finance.slippage.volatility_volume_share import VolatilityVolumeShare
from ziplime.gens.domain.simulation_clock import SimulationClock
from ziplime.exchanges.exchange import Exchange
from ziplime.exchanges.simulation_exchange import SimulationExchange
from ziplime.utils.run_algo import run_algorithm
import polars as pl


async def run_simulation(
        start_date: datetime.datetime,
        end_date: datetime.datetime,
        trading_calendar: str,
        emission_rate: datetime.timedelta,
        total_cash: float,
        market_data_source: DataSource,
        custom_data_sources: list[DataSource],
        algorithm_file: str,
        stop_on_error: bool,
        asset_service: AssetService,
        exchange: Exchange = None,
        default_exchange_name: str = "LIME",
        config_file: str | None = None,
        benchmark_asset_symbol: str | None = None,
        benchmark_returns: pl.Series | None = None,
        equity_commission: EquityCommissionModel | None = None,
        future_commission: FutureCommissionModel | None = None
):
    """
    Run a trading algorithm simulation within a defined time period and trading environment.

    This function initializes the simulation's trading calendar, algorithm file, simulation
    clock, exchange, and other associated components. It sets up the runtime environment and
    then executes the algorithm asynchronously, using the provided market data source, custom
    data sources, and optional benchmark data.

    Args:
        start_date (datetime.datetime): The datetime marking the start of the simulation.
        end_date (datetime.datetime): The datetime marking the end of the simulation.
        trading_calendar (str): The identifier for the trading calendar to use.
        emission_rate (datetime.timedelta): The frequency of emission for simulation data.
        total_cash (float): The starting cash balance for the simulation account.
        market_data_source (DataSource): The primary market data source to use in the simulation.
        custom_data_sources (list[DataSource]): List of custom data sources for additional data.
        algorithm_file (str): Path to the Python file containing the trading algorithm.
        stop_on_error (bool): Whether the simulation should stop on encountering an error.
        exchange (Exchange, optional): Exchange instance to use for the simulation. Defaults to None.
        config_file (str | None, optional): Path to the configuration file for the algorithm. Defaults to None.
        benchmark_asset_symbol (str | None, optional): Symbol for an asset to use as the benchmark. Defaults to None.
        benchmark_returns (pl.Series | None, optional): Custom benchmark returns to use for evaluation. Defaults to None.
        asset_service (AssetService): Service for managing assets.
        equity_commission: Model used to calculate fees when trading equities.
                           If not specified, ziplime.finance.commission.PerShare model is used with a default
                           share cost of 0.001 and minimum cost of trade 0.00
       future_commission: Model used to calculate fees when trading futures.
                          If not specified, ziplime.finance.commission.PerContract model is used with a default
                          cost per contract of 0.85 and minimum cost of trade 0.00


    Returns:
        Coroutine: The coroutine to execute the simulation and produce output results.
    """
    calendar = get_calendar(trading_calendar)

    algo = AlgorithmFile(algorithm_file=algorithm_file, algorithm_config_file=config_file)

    clock = SimulationClock(
        trading_calendar=calendar,
        start_date=start_date,
        end_date=end_date,
        emission_rate=emission_rate,
    )
    if equity_commission is None:
        equity_commission = PerShare(
            cost=DEFAULT_PER_SHARE_COST,
            min_trade_cost=DEFAULT_MINIMUM_COST_PER_EQUITY_TRADE,
        )
    if future_commission is None:
        future_commission = PerContract(
            cost=DEFAULT_PER_CONTRACT_COST,
            exchange_fee=FUTURE_EXCHANGE_FEES_BY_SYMBOL,
            min_trade_cost=DEFAULT_MINIMUM_COST_PER_FUTURE_TRADE
        )

    if exchange is None:
        exchange = SimulationExchange(
            name=default_exchange_name,
            country_code="US",
            trading_calendar=calendar,
            data_bundle=market_data_source,
            equity_slippage=FixedBasisPointsSlippage(),
            equity_commission=equity_commission,
            future_slippage=VolatilityVolumeShare(
                volume_limit=DEFAULT_FUTURE_VOLUME_SLIPPAGE_BAR_LIMIT,
            ),
            future_commission=future_commission,
            cash_balance=total_cash,
            clock=clock
        )

    return await run_algorithm(
        algorithm=algo,
        asset_service=asset_service,
        print_algo=True,
        metrics_set=default_metrics(),
        custom_loader=None,
        exchanges=[exchange],
        clock=clock,
        benchmark_returns=benchmark_returns,
        benchmark_asset_symbol=benchmark_asset_symbol,
        stop_on_error=stop_on_error,
        custom_data_sources=custom_data_sources,
    )



import datetime

import polars as pl
from abc import abstractmethod
from typing import Any, Self

from ziplime.constants.period import Period
from ziplime.data.domain.data_bundle import DataBundle


class BundleStorage:

    @abstractmethod
    async def store_bundle(self, data_bundle: DataBundle):
        """
        Method for storing a data bundle asynchronously.

        Args:
            data_bundle (DataBundle): The data bundle to be stored.
        """
        ...

    @abstractmethod
    async def load_data_bundle(self, data_bundle: DataBundle,
                               symbols: list[str] | None = None,
                               start_date: datetime.datetime | None = None,
                               end_date: datetime.datetime | None = None,
                               frequency: datetime.timedelta | Period | None = None,
                               start_auction_delta: datetime.timedelta = None,
                               end_auction_delta: datetime.timedelta = None,
                               aggregations: list[pl.Expr] = None,
                               ) -> pl.DataFrame:
        """
        Loads data from a DataBundle based on the specified parameters and returns
        a Polars DataFrame.

        Args:
            data_bundle (DataBundle): The source bundle containing the data to be loaded.
            symbols (list[str] | None, optional): A list of symbols to filter and load data for. If not
                provided, data for all available symbols is loaded.
            start_date (datetime.datetime | None, optional): The starting date-time for the data to be loaded.
                If not specified, it loads data from the earliest available date-time.
            end_date (datetime.datetime | None, optional): The ending date-time for the data to be loaded. If
                not specified, it loads data up to the latest available date-time.
            frequency (datetime.timedelta | Period | None, optional): The data sampling frequency for the
                loaded data. Defaults to the finest granularity if not specified.
            start_auction_delta (datetime.timedelta):
               Used when requested frequency is greater than ingested frequency.
               It allows defining custom start time for each frequency group.
               Example:
                 Ingested frequenct is 1m, requested frequency is 1d and start_auction_delta is 1h.
                 Before grouping data by 1d frequency, data will be filtered to include only data in each group that
                 is greater than 1h after the start of the group.
                 Can be useful if you want to test strategy on 1d frequenct but when running algorithm callback each day
                 1 hour after opening time
            end_auction_delta (datetime.timedelta):
                Used when requested frequency is greater than ingested frequency.
                It allows defining custom start time for each frequency group.
                Example:
                  Ingested frequenct is 1m, requested frequency is 1d and end_auction_delta is 1h.
                  Before grouping data by 1d frequency, data will be filtered to include only data in each group that
                  is lower than 1h before the end of the group.
                  Useful if you want to test strategy on 1d frequenct but when running algorithm callback each day
                  1 hour before closing time
            aggregations (list[pl.Expr], optional): A list of Polars aggregation expressions to be applied
                during the data loading process. Defaults to `None`.

        Returns:
            pl.DataFrame: A Polars DataFrame containing the loaded and potentially filtered/aggregated data.

        Raises:
            NotImplementedError: Must be raised if this method is called directly from an abstract class.
        """
        ...

    @classmethod
    @abstractmethod
    async def from_json(cls, data: dict[str, Any]) -> Self:
        """
        Asynchronously create an instance of the class from JSON data.

        Args:
            data (dict[str, Any]): A dictionary containing JSON data
                used to construct the instance.

        Returns:
            Self: An instance of the class constructed from the provided
                JSON data.
        """
        ...

    @abstractmethod
    async def to_json(self, data_bundle: DataBundle) -> dict[str, Any]:
        """
        Converts a DataBundle instance to its JSON representation asynchronously.

        Args:
            data_bundle (DataBundle): The data bundle to be converted to JSON.

        Returns:
            dict[str, Any]: The JSON representation of the data bundle.
        """
        ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: grpc/tradeapi/v1/accounts/accounts_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'grpc/tradeapi/v1/accounts/accounts_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.type import decimal_pb2 as google_dot_type_dot_decimal__pb2
from google.type import interval_pb2 as google_dot_type_dot_interval__pb2
from google.type import money_pb2 as google_dot_type_dot_money__pb2
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1 import side_pb2 as grpc_dot_tradeapi_dot_v1_dot_side__pb2
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1 import trade_pb2 as grpc_dot_tradeapi_dot_v1_dot_trade__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0grpc/tradeapi/v1/accounts/accounts_service.proto\x12\x19grpc.tradeapi.v1.accounts\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/type/decimal.proto\x1a\x1agoogle/type/interval.proto\x1a\x17google/type/money.proto\x1a\x1bgrpc/tradeapi/v1/side.proto\x1a\x1cgrpc/tradeapi/v1/trade.proto\"\'\n\x11GetAccountRequest\x12\x12\n\naccount_id\x18\x01 \x01(\t\"\xb1\x03\n\x12GetAccountResponse\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\x12$\n\x06\x65quity\x18\x04 \x01(\x0b\x32\x14.google.type.Decimal\x12/\n\x11unrealized_profit\x18\x05 \x01(\x0b\x32\x14.google.type.Decimal\x12\x36\n\tpositions\x18\x06 \x03(\x0b\x32#.grpc.tradeapi.v1.accounts.Position\x12 \n\x04\x63\x61sh\x18\x07 \x03(\x0b\x32\x12.google.type.Money\x12\x35\n\x0cportfolio_mc\x18\x08 \x01(\x0b\x32\x1d.grpc.tradeapi.v1.accounts.MCH\x00\x12\x37\n\rportfolio_mct\x18\t \x01(\x0b\x32\x1e.grpc.tradeapi.v1.accounts.MCTH\x00\x12;\n\x0fportfolio_forts\x18\n \x01(\x0b\x32 .grpc.tradeapi.v1.accounts.FORTSH\x00\x42\x0b\n\tportfolio\"\x92\x01\n\x02MC\x12,\n\x0e\x61vailable_cash\x18\x01 \x01(\x0b\x32\x14.google.type.Decimal\x12,\n\x0einitial_margin\x18\x02 \x01(\x0b\x32\x14.google.type.Decimal\x12\x30\n\x12maintenance_margin\x18\x03 \x01(\x0b\x32\x14.google.type.Decimal\"\x05\n\x03MCT\"c\n\x05\x46ORTS\x12,\n\x0e\x61vailable_cash\x18\x01 \x01(\x0b\x32\x14.google.type.Decimal\x12,\n\x0emoney_reserved\x18\x02 \x01(\x0b\x32\x14.google.type.Decimal\"[\n\rTradesRequest\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\'\n\x08interval\x18\x03 \x01(\x0b\x32\x15.google.type.Interval\"@\n\x0eTradesResponse\x12.\n\x06trades\x18\x01 \x03(\x0b\x32\x1e.grpc.tradeapi.v1.AccountTrade\"a\n\x13TransactionsRequest\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\'\n\x08interval\x18\x03 \x01(\x0b\x32\x15.google.type.Interval\"T\n\x14TransactionsResponse\x12<\n\x0ctransactions\x18\x01 \x03(\x0b\x32&.grpc.tradeapi.v1.accounts.Transaction\"\xa5\x02\n\x08Position\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12&\n\x08quantity\x18\x02 \x01(\x0b\x32\x14.google.type.Decimal\x12+\n\raverage_price\x18\x03 \x01(\x0b\x32\x14.google.type.Decimal\x12+\n\rcurrent_price\x18\x04 \x01(\x0b\x32\x14.google.type.Decimal\x12\x30\n\x12maintenance_margin\x18\x05 \x01(\x0b\x32\x14.google.type.Decimal\x12\'\n\tdaily_pnl\x18\x06 \x01(\x0b\x32\x14.google.type.Decimal\x12,\n\x0eunrealized_pnl\x18\x07 \x01(\x0b\x32\x14.google.type.Decimal\"\x85\x05\n\x0bTransaction\x12\n\n\x02id\x18\x01 \x01(\t\x12\x14\n\x08\x63\x61tegory\x18\x02 \x01(\tB\x02\x18\x01\x12-\n\ttimestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06symbol\x18\x05 \x01(\t\x12\"\n\x06\x63hange\x18\x06 \x01(\x0b\x32\x12.google.type.Money\x12;\n\x05trade\x18\x07 \x01(\x0b\x32,.grpc.tradeapi.v1.accounts.Transaction.Trade\x12X\n\x14transaction_category\x18\x08 \x01(\x0e\x32:.grpc.tradeapi.v1.accounts.Transaction.TransactionCategory\x12\x18\n\x10transaction_name\x18\t \x01(\t\x1a\x80\x01\n\x05Trade\x12\"\n\x04size\x18\x01 \x01(\x0b\x32\x14.google.type.Decimal\x12#\n\x05price\x18\x02 \x01(\x0b\x32\x14.google.type.Decimal\x12.\n\x10\x61\x63\x63rued_interest\x18\x03 \x01(\x0b\x32\x14.google.type.Decimal\"\xbc\x01\n\x13TransactionCategory\x12\n\n\x06OTHERS\x10\x00\x12\x0b\n\x07\x44\x45POSIT\x10\x01\x12\x0c\n\x08WITHDRAW\x10\x02\x12\n\n\x06INCOME\x10\x05\x12\x0e\n\nCOMMISSION\x10\x07\x12\x07\n\x03TAX\x10\x08\x12\x0f\n\x0bINHERITANCE\x10\t\x12\x0c\n\x08TRANSFER\x10\x0b\x12\x18\n\x14\x43ONTRACT_TERMINATION\x10\x0c\x12\x0c\n\x08OUTCOMES\x10\r\x12\x08\n\x04\x46INE\x10\x0f\x12\x08\n\x04LOAN\x10\x13\x32\xcc\x03\n\x0f\x41\x63\x63ountsService\x12\x8c\x01\n\nGetAccount\x12,.grpc.tradeapi.v1.accounts.GetAccountRequest\x1a-.grpc.tradeapi.v1.accounts.GetAccountResponse\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/v1/accounts/{account_id}\x12\x87\x01\n\x06Trades\x12(.grpc.tradeapi.v1.accounts.TradesRequest\x1a).grpc.tradeapi.v1.accounts.TradesResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /v1/accounts/{account_id}/trades\x12\x9f\x01\n\x0cTransactions\x12..grpc.tradeapi.v1.accounts.TransactionsRequest\x1a/.grpc.tradeapi.v1.accounts.TransactionsResponse\".\x82\xd3\xe4\x93\x02(\x12&/v1/accounts/{account_id}/transactionsB*P\x01Z&trade_api/v1/accounts/accounts_serviceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc.tradeapi.v1.accounts.accounts_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001Z&trade_api/v1/accounts/accounts_service'
  _globals['_TRANSACTION'].fields_by_name['category']._loaded_options = None
  _globals['_TRANSACTION'].fields_by_name['category']._serialized_options = b'\030\001'
  _globals['_ACCOUNTSSERVICE'].methods_by_name['GetAccount']._loaded_options = None
  _globals['_ACCOUNTSSERVICE'].methods_by_name['GetAccount']._serialized_options = b'\202\323\344\223\002\033\022\031/v1/accounts/{account_id}'
  _globals['_ACCOUNTSSERVICE'].methods_by_name['Trades']._loaded_options = None
  _globals['_ACCOUNTSSERVICE'].methods_by_name['Trades']._serialized_options = b'\202\323\344\223\002\"\022 /v1/accounts/{account_id}/trades'
  _globals['_ACCOUNTSSERVICE'].methods_by_name['Transactions']._loaded_options = None
  _globals['_ACCOUNTSSERVICE'].methods_by_name['Transactions']._serialized_options = b'\202\323\344\223\002(\022&/v1/accounts/{account_id}/transactions'
  _globals['_GETACCOUNTREQUEST']._serialized_start=281
  _globals['_GETACCOUNTREQUEST']._serialized_end=320
  _globals['_GETACCOUNTRESPONSE']._serialized_start=323
  _globals['_GETACCOUNTRESPONSE']._serialized_end=756
  _globals['_MC']._serialized_start=759
  _globals['_MC']._serialized_end=905
  _globals['_MCT']._serialized_start=907
  _globals['_MCT']._serialized_end=912
  _globals['_FORTS']._serialized_start=914
  _globals['_FORTS']._serialized_end=1013
  _globals['_TRADESREQUEST']._serialized_start=1015
  _globals['_TRADESREQUEST']._serialized_end=1106
  _globals['_TRADESRESPONSE']._serialized_start=1108
  _globals['_TRADESRESPONSE']._serialized_end=1172
  _globals['_TRANSACTIONSREQUEST']._serialized_start=1174
  _globals['_TRANSACTIONSREQUEST']._serialized_end=1271
  _globals['_TRANSACTIONSRESPONSE']._serialized_start=1273
  _globals['_TRANSACTIONSRESPONSE']._serialized_end=1357
  _globals['_POSITION']._serialized_start=1360
  _globals['_POSITION']._serialized_end=1653
  _globals['_TRANSACTION']._serialized_start=1656
  _globals['_TRANSACTION']._serialized_end=2301
  _globals['_TRANSACTION_TRADE']._serialized_start=1982
  _globals['_TRANSACTION_TRADE']._serialized_end=2110
  _globals['_TRANSACTION_TRANSACTIONCATEGORY']._serialized_start=2113
  _globals['_TRANSACTION_TRANSACTIONCATEGORY']._serialized_end=2301
  _globals['_ACCOUNTSSERVICE']._serialized_start=2304
  _globals['_ACCOUNTSSERVICE']._serialized_end=2764
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: grpc/tradeapi/v1/orders/orders_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'grpc/tradeapi/v1/orders/orders_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.type import decimal_pb2 as google_dot_type_dot_decimal__pb2
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1 import side_pb2 as grpc_dot_tradeapi_dot_v1_dot_side__pb2
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1 import trade_pb2 as grpc_dot_tradeapi_dot_v1_dot_trade__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,grpc/tradeapi/v1/orders/orders_service.proto\x12\x17grpc.tradeapi.v1.orders\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/type/decimal.proto\x1a\x1bgrpc/tradeapi/v1/side.proto\x1a\x1cgrpc/tradeapi/v1/trade.proto\"\xb5\x02\n\x11OrderTradeRequest\x12\x41\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32\x31.grpc.tradeapi.v1.orders.OrderTradeRequest.Action\x12\x46\n\tdata_type\x18\x02 \x01(\x0e\x32\x33.grpc.tradeapi.v1.orders.OrderTradeRequest.DataType\x12\x12\n\naccount_id\x18\x03 \x01(\t\"6\n\x06\x41\x63tion\x12\x14\n\x10\x41\x43TION_SUBSCRIBE\x10\x00\x12\x16\n\x12\x41\x43TION_UNSUBSCRIBE\x10\x01\"I\n\x08\x44\x61taType\x12\x11\n\rDATA_TYPE_ALL\x10\x00\x12\x14\n\x10\x44\x41TA_TYPE_ORDERS\x10\x01\x12\x14\n\x10\x44\x41TA_TYPE_TRADES\x10\x02\"y\n\x12OrderTradeResponse\x12\x33\n\x06orders\x18\x01 \x03(\x0b\x32#.grpc.tradeapi.v1.orders.OrderState\x12.\n\x06trades\x18\x02 \x03(\x0b\x32\x1e.grpc.tradeapi.v1.AccountTrade\"7\n\x0fGetOrderRequest\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x10\n\x08order_id\x18\x02 \x01(\t\"\xfe\x03\n\x05Order\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t\x12&\n\x08quantity\x18\x03 \x01(\x0b\x32\x14.google.type.Decimal\x12$\n\x04side\x18\x04 \x01(\x0e\x32\x16.grpc.tradeapi.v1.Side\x12\x30\n\x04type\x18\x05 \x01(\x0e\x32\".grpc.tradeapi.v1.orders.OrderType\x12;\n\rtime_in_force\x18\x06 \x01(\x0e\x32$.grpc.tradeapi.v1.orders.TimeInForce\x12)\n\x0blimit_price\x18\x07 \x01(\x0b\x32\x14.google.type.Decimal\x12(\n\nstop_price\x18\x08 \x01(\x0b\x32\x14.google.type.Decimal\x12>\n\x0estop_condition\x18\t \x01(\x0e\x32&.grpc.tradeapi.v1.orders.StopCondition\x12*\n\x04legs\x18\n \x03(\x0b\x32\x1c.grpc.tradeapi.v1.orders.Leg\x12\x17\n\x0f\x63lient_order_id\x18\x0b \x01(\t\x12:\n\x0cvalid_before\x18\x0c \x01(\x0e\x32$.grpc.tradeapi.v1.orders.ValidBefore\"c\n\x03Leg\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12&\n\x08quantity\x18\x02 \x01(\x0b\x32\x14.google.type.Decimal\x12$\n\x04side\x18\x03 \x01(\x0e\x32\x16.grpc.tradeapi.v1.Side\"\xa5\x02\n\nOrderState\x12\x10\n\x08order_id\x18\x01 \x01(\t\x12\x0f\n\x07\x65xec_id\x18\x02 \x01(\t\x12\x34\n\x06status\x18\x03 \x01(\x0e\x32$.grpc.tradeapi.v1.orders.OrderStatus\x12-\n\x05order\x18\x04 \x01(\x0b\x32\x1e.grpc.tradeapi.v1.orders.Order\x12/\n\x0btransact_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\taccept_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0bwithdraw_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"#\n\rOrdersRequest\x12\x12\n\naccount_id\x18\x01 \x01(\t\"E\n\x0eOrdersResponse\x12\x33\n\x06orders\x18\x01 \x03(\x0b\x32#.grpc.tradeapi.v1.orders.OrderState\":\n\x12\x43\x61ncelOrderRequest\x12\x12\n\naccount_id\x18\x01 \x01(\t\x12\x10\n\x08order_id\x18\x02 \x01(\t*\x9e\x01\n\tOrderType\x12\x1a\n\x16ORDER_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11ORDER_TYPE_MARKET\x10\x01\x12\x14\n\x10ORDER_TYPE_LIMIT\x10\x02\x12\x13\n\x0fORDER_TYPE_STOP\x10\x03\x12\x19\n\x15ORDER_TYPE_STOP_LIMIT\x10\x04\x12\x18\n\x14ORDER_TYPE_MULTI_LEG\x10\x05*\x89\x02\n\x0bTimeInForce\x12\x1d\n\x19TIME_IN_FORCE_UNSPECIFIED\x10\x00\x12\x15\n\x11TIME_IN_FORCE_DAY\x10\x01\x12\"\n\x1eTIME_IN_FORCE_GOOD_TILL_CANCEL\x10\x02\x12$\n TIME_IN_FORCE_GOOD_TILL_CROSSING\x10\x03\x12\x15\n\x11TIME_IN_FORCE_EXT\x10\x04\x12\x19\n\x15TIME_IN_FORCE_ON_OPEN\x10\x05\x12\x1a\n\x16TIME_IN_FORCE_ON_CLOSE\x10\x06\x12\x15\n\x11TIME_IN_FORCE_IOC\x10\x07\x12\x15\n\x11TIME_IN_FORCE_FOK\x10\x08*i\n\rStopCondition\x12\x1e\n\x1aSTOP_CONDITION_UNSPECIFIED\x10\x00\x12\x1a\n\x16STOP_CONDITION_LAST_UP\x10\x01\x12\x1c\n\x18STOP_CONDITION_LAST_DOWN\x10\x02*\xe7\x06\n\x0bOrderStatus\x12\x1c\n\x18ORDER_STATUS_UNSPECIFIED\x10\x00\x12\x14\n\x10ORDER_STATUS_NEW\x10\x01\x12!\n\x1dORDER_STATUS_PARTIALLY_FILLED\x10\x02\x12\x17\n\x13ORDER_STATUS_FILLED\x10\x03\x12\x1d\n\x19ORDER_STATUS_DONE_FOR_DAY\x10\x04\x12\x19\n\x15ORDER_STATUS_CANCELED\x10\x05\x12\x19\n\x15ORDER_STATUS_REPLACED\x10\x06\x12\x1f\n\x1bORDER_STATUS_PENDING_CANCEL\x10\x07\x12\x19\n\x15ORDER_STATUS_REJECTED\x10\t\x12\x1a\n\x16ORDER_STATUS_SUSPENDED\x10\n\x12\x1c\n\x18ORDER_STATUS_PENDING_NEW\x10\x0b\x12\x18\n\x14ORDER_STATUS_EXPIRED\x10\r\x12\x17\n\x13ORDER_STATUS_FAILED\x10\x10\x12\x1b\n\x17ORDER_STATUS_FORWARDING\x10\x11\x12\x15\n\x11ORDER_STATUS_WAIT\x10\x12\x12!\n\x1dORDER_STATUS_DENIED_BY_BROKER\x10\x13\x12%\n!ORDER_STATUS_REJECTED_BY_EXCHANGE\x10\x14\x12\x19\n\x15ORDER_STATUS_WATCHING\x10\x15\x12\x19\n\x15ORDER_STATUS_EXECUTED\x10\x16\x12\x19\n\x15ORDER_STATUS_DISABLED\x10\x17\x12\x1a\n\x16ORDER_STATUS_LINK_WAIT\x10\x18\x12\x1e\n\x1aORDER_STATUS_SL_GUARD_TIME\x10\x1b\x12\x1c\n\x18ORDER_STATUS_SL_EXECUTED\x10\x1c\x12\x1e\n\x1aORDER_STATUS_SL_FORWARDING\x10\x1d\x12\x1e\n\x1aORDER_STATUS_TP_GUARD_TIME\x10\x1e\x12\x1c\n\x18ORDER_STATUS_TP_EXECUTED\x10\x1f\x12\x1e\n\x1aORDER_STATUS_TP_CORRECTION\x10 \x12\x1e\n\x1aORDER_STATUS_TP_FORWARDING\x10!\x12#\n\x1fORDER_STATUS_TP_CORR_GUARD_TIME\x10\"*\x8c\x01\n\x0bValidBefore\x12\x1c\n\x18VALID_BEFORE_UNSPECIFIED\x10\x00\x12\x1b\n\x17VALID_BEFORE_END_OF_DAY\x10\x01\x12!\n\x1dVALID_BEFORE_GOOD_TILL_CANCEL\x10\x02\x12\x1f\n\x1bVALID_BEFORE_GOOD_TILL_DATE\x10\x03\x32\xb4\x05\n\rOrdersService\x12~\n\nPlaceOrder\x12\x1e.grpc.tradeapi.v1.orders.Order\x1a#.grpc.tradeapi.v1.orders.OrderState\"+\x82\xd3\xe4\x93\x02%\" /v1/accounts/{account_id}/orders:\x01*\x12\x94\x01\n\x0b\x43\x61ncelOrder\x12+.grpc.tradeapi.v1.orders.CancelOrderRequest\x1a#.grpc.tradeapi.v1.orders.OrderState\"3\x82\xd3\xe4\x93\x02-*+/v1/accounts/{account_id}/orders/{order_id}\x12\x86\x01\n\tGetOrders\x12&.grpc.tradeapi.v1.orders.OrdersRequest\x1a\'.grpc.tradeapi.v1.orders.OrdersResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /v1/accounts/{account_id}/orders\x12\x8e\x01\n\x08GetOrder\x12(.grpc.tradeapi.v1.orders.GetOrderRequest\x1a#.grpc.tradeapi.v1.orders.OrderState\"3\x82\xd3\xe4\x93\x02-\x12+/v1/accounts/{account_id}/orders/{order_id}\x12r\n\x13SubscribeOrderTrade\x12*.grpc.tradeapi.v1.orders.OrderTradeRequest\x1a+.grpc.tradeapi.v1.orders.OrderTradeResponse(\x01\x30\x01\x42&P\x01Z\"trade_api/v1/orders/orders_serviceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc.tradeapi.v1.orders.orders_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001Z\"trade_api/v1/orders/orders_service'
  _globals['_ORDERSSERVICE'].methods_by_name['PlaceOrder']._loaded_options = None
  _globals['_ORDERSSERVICE'].methods_by_name['PlaceOrder']._serialized_options = b'\202\323\344\223\002%\" /v1/accounts/{account_id}/orders:\001*'
  _globals['_ORDERSSERVICE'].methods_by_name['CancelOrder']._loaded_options = None
  _globals['_ORDERSSERVICE'].methods_by_name['CancelOrder']._serialized_options = b'\202\323\344\223\002-*+/v1/accounts/{account_id}/orders/{order_id}'
  _globals['_ORDERSSERVICE'].methods_by_name['GetOrders']._loaded_options = None
  _globals['_ORDERSSERVICE'].methods_by_name['GetOrders']._serialized_options = b'\202\323\344\223\002\"\022 /v1/accounts/{account_id}/orders'
  _globals['_ORDERSSERVICE'].methods_by_name['GetOrder']._loaded_options = None
  _globals['_ORDERSSERVICE'].methods_by_name['GetOrder']._serialized_options = b'\202\323\344\223\002-\022+/v1/accounts/{account_id}/orders/{order_id}'
  _globals['_ORDERTYPE']._serialized_start=1793
  _globals['_ORDERTYPE']._serialized_end=1951
  _globals['_TIMEINFORCE']._serialized_start=1954
  _globals['_TIMEINFORCE']._serialized_end=2219
  _globals['_STOPCONDITION']._serialized_start=2221
  _globals['_STOPCONDITION']._serialized_end=2326
  _globals['_ORDERSTATUS']._serialized_start=2329
  _globals['_ORDERSTATUS']._serialized_end=3200
  _globals['_VALIDBEFORE']._serialized_start=3203
  _globals['_VALIDBEFORE']._serialized_end=3343
  _globals['_ORDERTRADEREQUEST']._serialized_start=223
  _globals['_ORDERTRADEREQUEST']._serialized_end=532
  _globals['_ORDERTRADEREQUEST_ACTION']._serialized_start=403
  _globals['_ORDERTRADEREQUEST_ACTION']._serialized_end=457
  _globals['_ORDERTRADEREQUEST_DATATYPE']._serialized_start=459
  _globals['_ORDERTRADEREQUEST_DATATYPE']._serialized_end=532
  _globals['_ORDERTRADERESPONSE']._serialized_start=534
  _globals['_ORDERTRADERESPONSE']._serialized_end=655
  _globals['_GETORDERREQUEST']._serialized_start=657
  _globals['_GETORDERREQUEST']._serialized_end=712
  _globals['_ORDER']._serialized_start=715
  _globals['_ORDER']._serialized_end=1225
  _globals['_LEG']._serialized_start=1227
  _globals['_LEG']._serialized_end=1326
  _globals['_ORDERSTATE']._serialized_start=1329
  _globals['_ORDERSTATE']._serialized_end=1622
  _globals['_ORDERSREQUEST']._serialized_start=1624
  _globals['_ORDERSREQUEST']._serialized_end=1659
  _globals['_ORDERSRESPONSE']._serialized_start=1661
  _globals['_ORDERSRESPONSE']._serialized_end=1730
  _globals['_CANCELORDERREQUEST']._serialized_start=1732
  _globals['_CANCELORDERREQUEST']._serialized_end=1790
  _globals['_ORDERSSERVICE']._serialized_start=3346
  _globals['_ORDERSSERVICE']._serialized_end=4038
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: grpc/tradeapi/v1/assets/assets_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'grpc/tradeapi/v1/assets/assets_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.type import date_pb2 as google_dot_type_dot_date__pb2
from google.type import decimal_pb2 as google_dot_type_dot_decimal__pb2
from google.type import interval_pb2 as google_dot_type_dot_interval__pb2
from google.type import money_pb2 as google_dot_type_dot_money__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,grpc/tradeapi/v1/assets/assets_service.proto\x12\x17grpc.tradeapi.v1.assets\x1a\x1cgoogle/api/annotations.proto\x1a\x16google/type/date.proto\x1a\x19google/type/decimal.proto\x1a\x1agoogle/type/interval.proto\x1a\x17google/type/money.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x12\n\x10\x45xchangesRequest\"I\n\x11\x45xchangesResponse\x12\x34\n\texchanges\x18\x01 \x03(\x0b\x32!.grpc.tradeapi.v1.assets.Exchange\"\x0f\n\rAssetsRequest\"@\n\x0e\x41ssetsResponse\x12.\n\x06\x61ssets\x18\x01 \x03(\x0b\x32\x1e.grpc.tradeapi.v1.assets.Asset\"5\n\x0fGetAssetRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\"\xec\x01\n\x10GetAssetResponse\x12\r\n\x05\x62oard\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0e\n\x06ticker\x18\x03 \x01(\t\x12\x0b\n\x03mic\x18\x04 \x01(\t\x12\x0c\n\x04isin\x18\x05 \x01(\t\x12\x0c\n\x04type\x18\x06 \x01(\t\x12\x0c\n\x04name\x18\x07 \x01(\t\x12\x10\n\x08\x64\x65\x63imals\x18\n \x01(\x05\x12\x10\n\x08min_step\x18\x0b \x01(\x03\x12&\n\x08lot_size\x18\t \x01(\x0b\x32\x14.google.type.Decimal\x12*\n\x0f\x65xpiration_date\x18\x0c \x01(\x0b\x32\x11.google.type.Date\";\n\x15GetAssetParamsRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\"\xde\x03\n\x16GetAssetParamsResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\x12\x11\n\ttradeable\x18\x03 \x01(\x08\x12\x33\n\x08longable\x18\x04 \x01(\x0b\x32!.grpc.tradeapi.v1.assets.Longable\x12\x35\n\tshortable\x18\x05 \x01(\x0b\x32\".grpc.tradeapi.v1.assets.Shortable\x12,\n\x0elong_risk_rate\x18\x06 \x01(\x0b\x32\x14.google.type.Decimal\x12/\n\x0flong_collateral\x18\x07 \x01(\x0b\x32\x12.google.type.MoneyB\x02\x18\x01\x12-\n\x0fshort_risk_rate\x18\x08 \x01(\x0b\x32\x14.google.type.Decimal\x12\x30\n\x10short_collateral\x18\t \x01(\x0b\x32\x12.google.type.MoneyB\x02\x18\x01\x12/\n\x13long_initial_margin\x18\n \x01(\x0b\x32\x12.google.type.Money\x12\x30\n\x14short_initial_margin\x18\x0b \x01(\x0b\x32\x12.google.type.Money\"0\n\x13OptionsChainRequest\x12\x19\n\x11underlying_symbol\x18\x01 \x01(\t\"X\n\x14OptionsChainResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x30\n\x07options\x18\x02 \x03(\x0b\x32\x1f.grpc.tradeapi.v1.assets.Option\"!\n\x0fScheduleRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\"\xab\x01\n\x10ScheduleResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x44\n\x08sessions\x18\x02 \x03(\x0b\x32\x32.grpc.tradeapi.v1.assets.ScheduleResponse.Sessions\x1a\x41\n\x08Sessions\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\'\n\x08interval\x18\x02 \x01(\x0b\x32\x15.google.type.Interval\"\x0e\n\x0c\x43lockRequest\">\n\rClockResponse\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"%\n\x08\x45xchange\x12\x0b\n\x03mic\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"j\n\x05\x41sset\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0e\n\x06ticker\x18\x03 \x01(\t\x12\x0b\n\x03mic\x18\x04 \x01(\t\x12\x0c\n\x04isin\x18\x05 \x01(\t\x12\x0c\n\x04type\x18\x06 \x01(\t\x12\x0c\n\x04name\x18\x07 \x01(\t\"\xbc\x03\n\x06Option\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x32\n\x04type\x18\x02 \x01(\x0e\x32$.grpc.tradeapi.v1.assets.Option.Type\x12+\n\rcontract_size\x18\x04 \x01(\x0b\x32\x14.google.type.Decimal\x12*\n\x0ftrade_first_day\x18\x05 \x01(\x0b\x32\x11.google.type.Date\x12)\n\x0etrade_last_day\x18\x06 \x01(\x0b\x32\x11.google.type.Date\x12$\n\x06strike\x18\x07 \x01(\x0b\x32\x14.google.type.Decimal\x12(\n\nmultiplier\x18\t \x01(\x0b\x32\x14.google.type.Decimal\x12/\n\x14\x65xpiration_first_day\x18\n \x01(\x0b\x32\x11.google.type.Date\x12.\n\x13\x65xpiration_last_day\x18\x0b \x01(\x0b\x32\x11.google.type.Date\"9\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\r\n\tTYPE_CALL\x10\x01\x12\x0c\n\x08TYPE_PUT\x10\x02\"\x9e\x01\n\x08Longable\x12\x37\n\x05value\x18\x01 \x01(\x0e\x32(.grpc.tradeapi.v1.assets.Longable.Status\x12\x13\n\x0bhalted_days\x18\x02 \x01(\x05\"D\n\x06Status\x12\x11\n\rNOT_AVAILABLE\x10\x00\x12\r\n\tAVAILABLE\x10\x01\x12\x18\n\x14\x41\x43\x43OUNT_NOT_APPROVED\x10\x02\"\xc1\x01\n\tShortable\x12\x38\n\x05value\x18\x01 \x01(\x0e\x32).grpc.tradeapi.v1.assets.Shortable.Status\x12\x13\n\x0bhalted_days\x18\x02 \x01(\x05\"e\n\x06Status\x12\x11\n\rNOT_AVAILABLE\x10\x00\x12\r\n\tAVAILABLE\x10\x01\x12\x07\n\x03HTB\x10\x02\x12\x18\n\x14\x41\x43\x43OUNT_NOT_APPROVED\x10\x03\x12\x16\n\x12\x41VAILABLE_STRATEGY\x10\x04\x32\xa7\x07\n\rAssetsService\x12y\n\tExchanges\x12).grpc.tradeapi.v1.assets.ExchangesRequest\x1a*.grpc.tradeapi.v1.assets.ExchangesResponse\"\x15\x82\xd3\xe4\x93\x02\x0f\x12\r/v1/exchanges\x12m\n\x06\x41ssets\x12&.grpc.tradeapi.v1.assets.AssetsRequest\x1a\'.grpc.tradeapi.v1.assets.AssetsResponse\"\x12\x82\xd3\xe4\x93\x02\x0c\x12\n/v1/assets\x12|\n\x08GetAsset\x12(.grpc.tradeapi.v1.assets.GetAssetRequest\x1a).grpc.tradeapi.v1.assets.GetAssetResponse\"\x1b\x82\xd3\xe4\x93\x02\x15\x12\x13/v1/assets/{symbol}\x12\x95\x01\n\x0eGetAssetParams\x12..grpc.tradeapi.v1.assets.GetAssetParamsRequest\x1a/.grpc.tradeapi.v1.assets.GetAssetParamsResponse\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/v1/assets/{symbol}/params\x12\x9b\x01\n\x0cOptionsChain\x12,.grpc.tradeapi.v1.assets.OptionsChainRequest\x1a-.grpc.tradeapi.v1.assets.OptionsChainResponse\".\x82\xd3\xe4\x93\x02(\x12&/v1/assets/{underlying_symbol}/options\x12\x85\x01\n\x08Schedule\x12(.grpc.tradeapi.v1.assets.ScheduleRequest\x1a).grpc.tradeapi.v1.assets.ScheduleResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/v1/assets/{symbol}/schedule\x12p\n\x05\x43lock\x12%.grpc.tradeapi.v1.assets.ClockRequest\x1a&.grpc.tradeapi.v1.assets.ClockResponse\"\x18\x82\xd3\xe4\x93\x02\x12\x12\x10/v1/assets/clockB&P\x01Z\"trade_api/v1/assets/assets_serviceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc.tradeapi.v1.assets.assets_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001Z\"trade_api/v1/assets/assets_service'
  _globals['_GETASSETPARAMSRESPONSE'].fields_by_name['long_collateral']._loaded_options = None
  _globals['_GETASSETPARAMSRESPONSE'].fields_by_name['long_collateral']._serialized_options = b'\030\001'
  _globals['_GETASSETPARAMSRESPONSE'].fields_by_name['short_collateral']._loaded_options = None
  _globals['_GETASSETPARAMSRESPONSE'].fields_by_name['short_collateral']._serialized_options = b'\030\001'
  _globals['_ASSETSSERVICE'].methods_by_name['Exchanges']._loaded_options = None
  _globals['_ASSETSSERVICE'].methods_by_name['Exchanges']._serialized_options = b'\202\323\344\223\002\017\022\r/v1/exchanges'
  _globals['_ASSETSSERVICE'].methods_by_name['Assets']._loaded_options = None
  _globals['_ASSETSSERVICE'].methods_by_name['Assets']._serialized_options = b'\202\323\344\223\002\014\022\n/v1/assets'
  _globals['_ASSETSSERVICE'].methods_by_name['GetAsset']._loaded_options = None
  _globals['_ASSETSSERVICE'].methods_by_name['GetAsset']._serialized_options = b'\202\323\344\223\002\025\022\023/v1/assets/{symbol}'
  _globals['_ASSETSSERVICE'].methods_by_name['GetAssetParams']._loaded_options = None
  _globals['_ASSETSSERVICE'].methods_by_name['GetAssetParams']._serialized_options = b'\202\323\344\223\002\034\022\032/v1/assets/{symbol}/params'
  _globals['_ASSETSSERVICE'].methods_by_name['OptionsChain']._loaded_options = None
  _globals['_ASSETSSERVICE'].methods_by_name['OptionsChain']._serialized_options = b'\202\323\344\223\002(\022&/v1/assets/{underlying_symbol}/options'
  _globals['_ASSETSSERVICE'].methods_by_name['Schedule']._loaded_options = None
  _globals['_ASSETSSERVICE'].methods_by_name['Schedule']._serialized_options = b'\202\323\344\223\002\036\022\034/v1/assets/{symbol}/schedule'
  _globals['_ASSETSSERVICE'].methods_by_name['Clock']._loaded_options = None
  _globals['_ASSETSSERVICE'].methods_by_name['Clock']._serialized_options = b'\202\323\344\223\002\022\022\020/v1/assets/clock'
  _globals['_EXCHANGESREQUEST']._serialized_start=240
  _globals['_EXCHANGESREQUEST']._serialized_end=258
  _globals['_EXCHANGESRESPONSE']._serialized_start=260
  _globals['_EXCHANGESRESPONSE']._serialized_end=333
  _globals['_ASSETSREQUEST']._serialized_start=335
  _globals['_ASSETSREQUEST']._serialized_end=350
  _globals['_ASSETSRESPONSE']._serialized_start=352
  _globals['_ASSETSRESPONSE']._serialized_end=416
  _globals['_GETASSETREQUEST']._serialized_start=418
  _globals['_GETASSETREQUEST']._serialized_end=471
  _globals['_GETASSETRESPONSE']._serialized_start=474
  _globals['_GETASSETRESPONSE']._serialized_end=710
  _globals['_GETASSETPARAMSREQUEST']._serialized_start=712
  _globals['_GETASSETPARAMSREQUEST']._serialized_end=771
  _globals['_GETASSETPARAMSRESPONSE']._serialized_start=774
  _globals['_GETASSETPARAMSRESPONSE']._serialized_end=1252
  _globals['_OPTIONSCHAINREQUEST']._serialized_start=1254
  _globals['_OPTIONSCHAINREQUEST']._serialized_end=1302
  _globals['_OPTIONSCHAINRESPONSE']._serialized_start=1304
  _globals['_OPTIONSCHAINRESPONSE']._serialized_end=1392
  _globals['_SCHEDULEREQUEST']._serialized_start=1394
  _globals['_SCHEDULEREQUEST']._serialized_end=1427
  _globals['_SCHEDULERESPONSE']._serialized_start=1430
  _globals['_SCHEDULERESPONSE']._serialized_end=1601
  _globals['_SCHEDULERESPONSE_SESSIONS']._serialized_start=1536
  _globals['_SCHEDULERESPONSE_SESSIONS']._serialized_end=1601
  _globals['_CLOCKREQUEST']._serialized_start=1603
  _globals['_CLOCKREQUEST']._serialized_end=1617
  _globals['_CLOCKRESPONSE']._serialized_start=1619
  _globals['_CLOCKRESPONSE']._serialized_end=1681
  _globals['_EXCHANGE']._serialized_start=1683
  _globals['_EXCHANGE']._serialized_end=1720
  _globals['_ASSET']._serialized_start=1722
  _globals['_ASSET']._serialized_end=1828
  _globals['_OPTION']._serialized_start=1831
  _globals['_OPTION']._serialized_end=2275
  _globals['_OPTION_TYPE']._serialized_start=2218
  _globals['_OPTION_TYPE']._serialized_end=2275
  _globals['_LONGABLE']._serialized_start=2278
  _globals['_LONGABLE']._serialized_end=2436
  _globals['_LONGABLE_STATUS']._serialized_start=2368
  _globals['_LONGABLE_STATUS']._serialized_end=2436
  _globals['_SHORTABLE']._serialized_start=2439
  _globals['_SHORTABLE']._serialized_end=2632
  _globals['_SHORTABLE_STATUS']._serialized_start=2531
  _globals['_SHORTABLE_STATUS']._serialized_end=2632
  _globals['_ASSETSSERVICE']._serialized_start=2635
  _globals['_ASSETSSERVICE']._serialized_end=3570
# @@protoc_insertion_point(module_scope)



import os
from pathlib import Path

from ziplime.data.services.bundle_service import BundleService
from ziplime.data.services.file_system_bundle_registry import FileSystemBundleRegistry
from ziplime.data.services.lime_trader_sdk_data_source import LimeTraderSdkDataSource
from ziplime.exchanges.lime_trader_sdk.lime_trader_sdk_exchange import LimeTraderSdkExchange
from ziplime.data.data_sources.limex_hub_fundamental_data_source import LimexHubFundamentalDataSource


def get_data_source(code: str):
    if code == "lime-trader-sdk":
        lime_trader_sdk_credentials = os.environ.get("LIME_SDK_CREDENTIALS_FILE", None)
        if lime_trader_sdk_credentials is None:
            raise ValueError("Missing LIME_SDK_CREDENTIALS_FILE environment variable.")
        return LimeTraderSdkDataSource(lime_sdk_credentials_file=lime_trader_sdk_credentials)
    raise Exception("Unsupported live market data provider!")

def get_fundamental_data_provider(code: str):
    if code == "limex-hub":
        limex_hub_key = os.environ.get("LIMEX_API_KEY", None)
        maximum_threads = os.environ.get("LIMEX_HUB_MAXIMUM_THREADS", None)
        if limex_hub_key is None:
            raise ValueError("Missing LIMEX_API_KEY environment variable.")
        return LimexHubFundamentalDataSource(limex_api_key=limex_hub_key, maximum_threads=maximum_threads)
    raise Exception("Unsupported fundamental data provider!")

def get_exchange(code: str):
    if code == "lime-trader-sdk":
        lime_trader_sdk_credentials = os.environ.get("LIME_SDK_CREDENTIALS_FILE", None)
        if lime_trader_sdk_credentials is None:
            raise ValueError("Missing LIME_SDK_CREDENTIALS_FILE environment variable.")
        return LimeTraderSdkExchange(lime_sdk_credentials_file=lime_trader_sdk_credentials)
    raise Exception("Unsupported live market data provider!")


def get_bundle_service(bundle_storage_path: str = str(Path(Path.home(), ".ziplime", "data"))) -> BundleService:
    bundle_registry = FileSystemBundleRegistry(base_data_path=bundle_storage_path)
    bundle_service = BundleService(bundle_registry=bundle_registry)
    return bundle_service


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: grpc/tradeapi/v1/trade.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'grpc/tradeapi/v1/trade.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.type import decimal_pb2 as google_dot_type_dot_decimal__pb2
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1 import side_pb2 as grpc_dot_tradeapi_dot_v1_dot_side__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cgrpc/tradeapi/v1/trade.proto\x12\x10grpc.tradeapi.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/type/decimal.proto\x1a\x1bgrpc/tradeapi/v1/side.proto\"\xf4\x01\n\x0c\x41\x63\x63ountTrade\x12\x10\n\x08trade_id\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t\x12#\n\x05price\x18\x03 \x01(\x0b\x32\x14.google.type.Decimal\x12\"\n\x04size\x18\x04 \x01(\x0b\x32\x14.google.type.Decimal\x12$\n\x04side\x18\x05 \x01(\x0e\x32\x16.grpc.tradeapi.v1.Side\x12-\n\ttimestamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08order_id\x18\x07 \x01(\t\x12\x12\n\naccount_id\x18\x08 \x01(\tB\x16P\x01Z\x12trade_api/v1/tradeb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc.tradeapi.v1.trade_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001Z\022trade_api/v1/trade'
  _globals['_ACCOUNTTRADE']._serialized_start=140
  _globals['_ACCOUNTTRADE']._serialized_end=384
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: grpc/tradeapi/v1/marketdata/marketdata_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'grpc/tradeapi/v1/marketdata/marketdata_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.type import decimal_pb2 as google_dot_type_dot_decimal__pb2
from google.type import interval_pb2 as google_dot_type_dot_interval__pb2
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1 import side_pb2 as grpc_dot_tradeapi_dot_v1_dot_side__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4grpc/tradeapi/v1/marketdata/marketdata_service.proto\x12\x1bgrpc.tradeapi.v1.marketdata\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/type/decimal.proto\x1a\x1agoogle/type/interval.proto\x1a\x1bgrpc/tradeapi/v1/side.proto\"\x81\x01\n\x0b\x42\x61rsRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x39\n\ttimeframe\x18\x02 \x01(\x0e\x32&.grpc.tradeapi.v1.marketdata.TimeFrame\x12\'\n\x08interval\x18\x03 \x01(\x0b\x32\x15.google.type.Interval\"N\n\x0c\x42\x61rsResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12.\n\x04\x62\x61rs\x18\x02 \x03(\x0b\x32 .grpc.tradeapi.v1.marketdata.Bar\"\x1e\n\x0cQuoteRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\"R\n\rQuoteResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x31\n\x05quote\x18\x02 \x01(\x0b\x32\".grpc.tradeapi.v1.marketdata.Quote\"\"\n\x10OrderBookRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\"^\n\x11OrderBookResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x39\n\torderbook\x18\x02 \x01(\x0b\x32&.grpc.tradeapi.v1.marketdata.OrderBook\"%\n\x13LatestTradesRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\"Z\n\x14LatestTradesResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x32\n\x06trades\x18\x02 \x03(\x0b\x32\".grpc.tradeapi.v1.marketdata.Trade\"(\n\x15SubscribeQuoteRequest\x12\x0f\n\x07symbols\x18\x01 \x03(\t\"\x84\x01\n\x16SubscribeQuoteResponse\x12\x31\n\x05quote\x18\x01 \x03(\x0b\x32\".grpc.tradeapi.v1.marketdata.Quote\x12\x37\n\x05\x65rror\x18\x02 \x01(\x0b\x32(.grpc.tradeapi.v1.marketdata.StreamError\"+\n\x19SubscribeOrderBookRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\"^\n\x1aSubscribeOrderBookResponse\x12@\n\norder_book\x18\x01 \x03(\x0b\x32,.grpc.tradeapi.v1.marketdata.StreamOrderBook\"a\n\x14SubscribeBarsRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x39\n\ttimeframe\x18\x02 \x01(\x0e\x32&.grpc.tradeapi.v1.marketdata.TimeFrame\"W\n\x15SubscribeBarsResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12.\n\x04\x62\x61rs\x18\x02 \x03(\x0b\x32 .grpc.tradeapi.v1.marketdata.Bar\"\xea\x01\n\x03\x42\x61r\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\"\n\x04open\x18\x02 \x01(\x0b\x32\x14.google.type.Decimal\x12\"\n\x04high\x18\x03 \x01(\x0b\x32\x14.google.type.Decimal\x12!\n\x03low\x18\x04 \x01(\x0b\x32\x14.google.type.Decimal\x12#\n\x05\x63lose\x18\x05 \x01(\x0b\x32\x14.google.type.Decimal\x12$\n\x06volume\x18\x06 \x01(\x0b\x32\x14.google.type.Decimal\"\xc8\x07\n\x05Quote\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12!\n\x03\x61sk\x18\x03 \x01(\x0b\x32\x14.google.type.Decimal\x12&\n\x08\x61sk_size\x18\x04 \x01(\x0b\x32\x14.google.type.Decimal\x12!\n\x03\x62id\x18\x05 \x01(\x0b\x32\x14.google.type.Decimal\x12&\n\x08\x62id_size\x18\x06 \x01(\x0b\x32\x14.google.type.Decimal\x12\"\n\x04last\x18\x07 \x01(\x0b\x32\x14.google.type.Decimal\x12\'\n\tlast_size\x18\x08 \x01(\x0b\x32\x14.google.type.Decimal\x12$\n\x06volume\x18\t \x01(\x0b\x32\x14.google.type.Decimal\x12&\n\x08turnover\x18\n \x01(\x0b\x32\x14.google.type.Decimal\x12\"\n\x04open\x18\x0b \x01(\x0b\x32\x14.google.type.Decimal\x12\"\n\x04high\x18\x0c \x01(\x0b\x32\x14.google.type.Decimal\x12!\n\x03low\x18\r \x01(\x0b\x32\x14.google.type.Decimal\x12#\n\x05\x63lose\x18\x0e \x01(\x0b\x32\x14.google.type.Decimal\x12$\n\x06\x63hange\x18\x0f \x01(\x0b\x32\x14.google.type.Decimal\x12;\n\x06option\x18\x32 \x01(\x0b\x32).grpc.tradeapi.v1.marketdata.Quote.OptionH\x00\x1a\xce\x02\n\x06Option\x12+\n\ropen_interest\x18\x01 \x01(\x0b\x32\x14.google.type.Decimal\x12\x30\n\x12implied_volatility\x18\x02 \x01(\x0b\x32\x14.google.type.Decimal\x12/\n\x11theoretical_price\x18\x03 \x01(\x0b\x32\x14.google.type.Decimal\x12#\n\x05\x64\x65lta\x18\x04 \x01(\x0b\x32\x14.google.type.Decimal\x12#\n\x05gamma\x18\x05 \x01(\x0b\x32\x14.google.type.Decimal\x12#\n\x05theta\x18\x06 \x01(\x0b\x32\x14.google.type.Decimal\x12\"\n\x04vega\x18\x07 \x01(\x0b\x32\x14.google.type.Decimal\x12!\n\x03rho\x18\x08 \x01(\x0b\x32\x14.google.type.DecimalB\x0b\n\tadditions\"\xa7\x03\n\tOrderBook\x12\x38\n\x04rows\x18\x01 \x03(\x0b\x32*.grpc.tradeapi.v1.marketdata.OrderBook.Row\x1a\xdf\x02\n\x03Row\x12#\n\x05price\x18\x01 \x01(\x0b\x32\x14.google.type.Decimal\x12)\n\tsell_size\x18\x02 \x01(\x0b\x32\x14.google.type.DecimalH\x00\x12(\n\x08\x62uy_size\x18\x03 \x01(\x0b\x32\x14.google.type.DecimalH\x00\x12\x41\n\x06\x61\x63tion\x18\x04 \x01(\x0e\x32\x31.grpc.tradeapi.v1.marketdata.OrderBook.Row.Action\x12\x0c\n\x04mpid\x18\x05 \x01(\t\x12-\n\ttimestamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"V\n\x06\x41\x63tion\x12\x16\n\x12\x41\x43TION_UNSPECIFIED\x10\x00\x12\x11\n\rACTION_REMOVE\x10\x01\x12\x0e\n\nACTION_ADD\x10\x02\x12\x11\n\rACTION_UPDATE\x10\x03\x42\x06\n\x04side\"\xc5\x01\n\x05Trade\x12\x10\n\x08trade_id\x18\x01 \x01(\t\x12\x0c\n\x04mpid\x18\x02 \x01(\t\x12-\n\ttimestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12#\n\x05price\x18\x04 \x01(\x0b\x32\x14.google.type.Decimal\x12\"\n\x04size\x18\x05 \x01(\x0b\x32\x14.google.type.Decimal\x12$\n\x04side\x18\x06 \x01(\x0e\x32\x16.grpc.tradeapi.v1.Side\"0\n\x0bStreamError\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\"\xc9\x03\n\x0fStreamOrderBook\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12>\n\x04rows\x18\x02 \x03(\x0b\x32\x30.grpc.tradeapi.v1.marketdata.StreamOrderBook.Row\x1a\xe5\x02\n\x03Row\x12#\n\x05price\x18\x01 \x01(\x0b\x32\x14.google.type.Decimal\x12)\n\tsell_size\x18\x02 \x01(\x0b\x32\x14.google.type.DecimalH\x00\x12(\n\x08\x62uy_size\x18\x03 \x01(\x0b\x32\x14.google.type.DecimalH\x00\x12G\n\x06\x61\x63tion\x18\x04 \x01(\x0e\x32\x37.grpc.tradeapi.v1.marketdata.StreamOrderBook.Row.Action\x12\x0c\n\x04mpid\x18\x05 \x01(\t\x12-\n\ttimestamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"V\n\x06\x41\x63tion\x12\x16\n\x12\x41\x43TION_UNSPECIFIED\x10\x00\x12\x11\n\rACTION_REMOVE\x10\x01\x12\x0e\n\nACTION_ADD\x10\x02\x12\x11\n\rACTION_UPDATE\x10\x03\x42\x06\n\x04side\".\n\x1cSubscribeLatestTradesRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\"c\n\x1dSubscribeLatestTradesResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x32\n\x06trades\x18\x02 \x03(\x0b\x32\".grpc.tradeapi.v1.marketdata.Trade*\x8b\x02\n\tTimeFrame\x12\x1a\n\x16TIME_FRAME_UNSPECIFIED\x10\x00\x12\x11\n\rTIME_FRAME_M1\x10\x01\x12\x11\n\rTIME_FRAME_M5\x10\x05\x12\x12\n\x0eTIME_FRAME_M15\x10\t\x12\x12\n\x0eTIME_FRAME_M30\x10\x0b\x12\x11\n\rTIME_FRAME_H1\x10\x0c\x12\x11\n\rTIME_FRAME_H2\x10\r\x12\x11\n\rTIME_FRAME_H4\x10\x0f\x12\x11\n\rTIME_FRAME_H8\x10\x11\x12\x10\n\x0cTIME_FRAME_D\x10\x13\x12\x10\n\x0cTIME_FRAME_W\x10\x14\x12\x11\n\rTIME_FRAME_MN\x10\x15\x12\x11\n\rTIME_FRAME_QR\x10\x16\x32\x80\t\n\x11MarketDataService\x12\x82\x01\n\x04\x42\x61rs\x12(.grpc.tradeapi.v1.marketdata.BarsRequest\x1a).grpc.tradeapi.v1.marketdata.BarsResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/v1/instruments/{symbol}/bars\x12\x92\x01\n\tLastQuote\x12).grpc.tradeapi.v1.marketdata.QuoteRequest\x1a*.grpc.tradeapi.v1.marketdata.QuoteResponse\".\x82\xd3\xe4\x93\x02(\x12&/v1/instruments/{symbol}/quotes/latest\x12\x96\x01\n\tOrderBook\x12-.grpc.tradeapi.v1.marketdata.OrderBookRequest\x1a..grpc.tradeapi.v1.marketdata.OrderBookResponse\"*\x82\xd3\xe4\x93\x02$\x12\"/v1/instruments/{symbol}/orderbook\x12\xa3\x01\n\x0cLatestTrades\x12\x30.grpc.tradeapi.v1.marketdata.LatestTradesRequest\x1a\x31.grpc.tradeapi.v1.marketdata.LatestTradesResponse\".\x82\xd3\xe4\x93\x02(\x12&/v1/instruments/{symbol}/trades/latest\x12{\n\x0eSubscribeQuote\x12\x32.grpc.tradeapi.v1.marketdata.SubscribeQuoteRequest\x1a\x33.grpc.tradeapi.v1.marketdata.SubscribeQuoteResponse0\x01\x12\x87\x01\n\x12SubscribeOrderBook\x12\x36.grpc.tradeapi.v1.marketdata.SubscribeOrderBookRequest\x1a\x37.grpc.tradeapi.v1.marketdata.SubscribeOrderBookResponse0\x01\x12\x90\x01\n\x15SubscribeLatestTrades\x12\x39.grpc.tradeapi.v1.marketdata.SubscribeLatestTradesRequest\x1a:.grpc.tradeapi.v1.marketdata.SubscribeLatestTradesResponse0\x01\x12x\n\rSubscribeBars\x12\x31.grpc.tradeapi.v1.marketdata.SubscribeBarsRequest\x1a\x32.grpc.tradeapi.v1.marketdata.SubscribeBarsResponse0\x01\x42.P\x01Z*trade_api/v1/marketdata/marketdata_serviceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc.tradeapi.v1.marketdata.marketdata_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'P\001Z*trade_api/v1/marketdata/marketdata_service'
  _globals['_MARKETDATASERVICE'].methods_by_name['Bars']._loaded_options = None
  _globals['_MARKETDATASERVICE'].methods_by_name['Bars']._serialized_options = b'\202\323\344\223\002\037\022\035/v1/instruments/{symbol}/bars'
  _globals['_MARKETDATASERVICE'].methods_by_name['LastQuote']._loaded_options = None
  _globals['_MARKETDATASERVICE'].methods_by_name['LastQuote']._serialized_options = b'\202\323\344\223\002(\022&/v1/instruments/{symbol}/quotes/latest'
  _globals['_MARKETDATASERVICE'].methods_by_name['OrderBook']._loaded_options = None
  _globals['_MARKETDATASERVICE'].methods_by_name['OrderBook']._serialized_options = b'\202\323\344\223\002$\022\"/v1/instruments/{symbol}/orderbook'
  _globals['_MARKETDATASERVICE'].methods_by_name['LatestTrades']._loaded_options = None
  _globals['_MARKETDATASERVICE'].methods_by_name['LatestTrades']._serialized_options = b'\202\323\344\223\002(\022&/v1/instruments/{symbol}/trades/latest'
  _globals['_TIMEFRAME']._serialized_start=3823
  _globals['_TIMEFRAME']._serialized_end=4090
  _globals['_BARSREQUEST']._serialized_start=233
  _globals['_BARSREQUEST']._serialized_end=362
  _globals['_BARSRESPONSE']._serialized_start=364
  _globals['_BARSRESPONSE']._serialized_end=442
  _globals['_QUOTEREQUEST']._serialized_start=444
  _globals['_QUOTEREQUEST']._serialized_end=474
  _globals['_QUOTERESPONSE']._serialized_start=476
  _globals['_QUOTERESPONSE']._serialized_end=558
  _globals['_ORDERBOOKREQUEST']._serialized_start=560
  _globals['_ORDERBOOKREQUEST']._serialized_end=594
  _globals['_ORDERBOOKRESPONSE']._serialized_start=596
  _globals['_ORDERBOOKRESPONSE']._serialized_end=690
  _globals['_LATESTTRADESREQUEST']._serialized_start=692
  _globals['_LATESTTRADESREQUEST']._serialized_end=729
  _globals['_LATESTTRADESRESPONSE']._serialized_start=731
  _globals['_LATESTTRADESRESPONSE']._serialized_end=821
  _globals['_SUBSCRIBEQUOTEREQUEST']._serialized_start=823
  _globals['_SUBSCRIBEQUOTEREQUEST']._serialized_end=863
  _globals['_SUBSCRIBEQUOTERESPONSE']._serialized_start=866
  _globals['_SUBSCRIBEQUOTERESPONSE']._serialized_end=998
  _globals['_SUBSCRIBEORDERBOOKREQUEST']._serialized_start=1000
  _globals['_SUBSCRIBEORDERBOOKREQUEST']._serialized_end=1043
  _globals['_SUBSCRIBEORDERBOOKRESPONSE']._serialized_start=1045
  _globals['_SUBSCRIBEORDERBOOKRESPONSE']._serialized_end=1139
  _globals['_SUBSCRIBEBARSREQUEST']._serialized_start=1141
  _globals['_SUBSCRIBEBARSREQUEST']._serialized_end=1238
  _globals['_SUBSCRIBEBARSRESPONSE']._serialized_start=1240
  _globals['_SUBSCRIBEBARSRESPONSE']._serialized_end=1327
  _globals['_BAR']._serialized_start=1330
  _globals['_BAR']._serialized_end=1564
  _globals['_QUOTE']._serialized_start=1567
  _globals['_QUOTE']._serialized_end=2535
  _globals['_QUOTE_OPTION']._serialized_start=2188
  _globals['_QUOTE_OPTION']._serialized_end=2522
  _globals['_ORDERBOOK']._serialized_start=2538
  _globals['_ORDERBOOK']._serialized_end=2961
  _globals['_ORDERBOOK_ROW']._serialized_start=2610
  _globals['_ORDERBOOK_ROW']._serialized_end=2961
  _globals['_ORDERBOOK_ROW_ACTION']._serialized_start=2867
  _globals['_ORDERBOOK_ROW_ACTION']._serialized_end=2953
  _globals['_TRADE']._serialized_start=2964
  _globals['_TRADE']._serialized_end=3161
  _globals['_STREAMERROR']._serialized_start=3163
  _globals['_STREAMERROR']._serialized_end=3211
  _globals['_STREAMORDERBOOK']._serialized_start=3214
  _globals['_STREAMORDERBOOK']._serialized_end=3671
  _globals['_STREAMORDERBOOK_ROW']._serialized_start=3314
  _globals['_STREAMORDERBOOK_ROW']._serialized_end=3671
  _globals['_STREAMORDERBOOK_ROW_ACTION']._serialized_start=2867
  _globals['_STREAMORDERBOOK_ROW_ACTION']._serialized_end=2953
  _globals['_SUBSCRIBELATESTTRADESREQUEST']._serialized_start=3673
  _globals['_SUBSCRIBELATESTTRADESREQUEST']._serialized_end=3719
  _globals['_SUBSCRIBELATESTTRADESRESPONSE']._serialized_start=3721
  _globals['_SUBSCRIBELATESTTRADESRESPONSE']._serialized_end=3820
  _globals['_MARKETDATASERVICE']._serialized_start=4093
  _globals['_MARKETDATASERVICE']._serialized_end=5245
# @@protoc_insertion_point(module_scope)



import datetime
import multiprocessing
import os
from typing import Self

import limexhub
import structlog

import polars as pl

from ziplime.assets.entities.asset import Asset
from ziplime.assets.entities.equity import Equity
from ziplime.assets.entities.equity_symbol_mapping import EquitySymbolMapping
from ziplime.assets.models.exchange_info import ExchangeInfo
from ziplime.data.data_sources.asset_data_source import AssetDataSource


class LimexHubAssetDataSource(AssetDataSource):
    def __init__(self, limex_api_key: str, maximum_threads: int | None = None):
        super().__init__()
        self._limex_api_key = limex_api_key
        self._logger = structlog.get_logger(__name__)
        self._limex_client = limexhub.RestAPI(token=limex_api_key)
        if maximum_threads is not None:
            self._maximum_threads = min(multiprocessing.cpu_count() * 2, maximum_threads)
        else:
            self._maximum_threads = multiprocessing.cpu_count() * 2

    async def get_assets(self, **kwargs) -> list[Asset]:
        assets = self._limex_client.instruments()

        assets_df = pl.from_dataframe(assets)
        assets_df = assets_df.rename({
            "ticker": "symbol"
        })
        asset_start_date = datetime.datetime(year=1900, month=1, day=1, tzinfo=datetime.timezone.utc)
        asset_end_date = datetime.datetime(year=2099, month=1, day=1, tzinfo=datetime.timezone.utc)

        equities = [
            Equity(
                asset_name=asset["symbol"],
                symbol_mapping={
                    "LIME": EquitySymbolMapping(
                        symbol=asset["symbol"],
                        exchange_name="LIME",
                        start_date=asset_start_date,
                        end_date=asset_end_date,
                        company_symbol="",
                        share_class_symbol=""
                    )
                },
                sid=None,
                start_date=asset_start_date,
                end_date=asset_end_date,
                auto_close_date=asset_end_date,
                first_traded=asset_start_date,
                mic="LIME"
            ) for asset in assets_df.iter_rows(named=True)
        ]

        return equities

    async def get_exchanges(self, **kwargs) -> list[ExchangeInfo]:
        exchanges = [ExchangeInfo(exchange="LIME", canonical_name="LIME", country_code="US")]
        return exchanges

    async def get_constituents(self, index: str) -> pl.DataFrame:
        assets = self._limex_client.constituents(index)
        return assets

    @classmethod
    def from_env(cls) -> Self:
        limex_hub_key = os.environ.get("LIMEX_API_KEY", None)
        maximum_threads = os.environ.get("LIMEX_HUB_MAXIMUM_THREADS", None)
        if limex_hub_key is None:
            raise ValueError("Missing LIMEX_API_KEY environment variable.")
        return cls(limex_api_key=limex_hub_key, maximum_threads=maximum_threads)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1.assets import assets_service_pb2 as grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2

GRPC_GENERATED_VERSION = '1.74.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in grpc/tradeapi/v1/assets/assets_service_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class AssetsServiceStub(object):
    def __init__(self, channel):
        self.Exchanges = channel.unary_unary(
                '/grpc.tradeapi.v1.assets.AssetsService/Exchanges',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ExchangesRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ExchangesResponse.FromString,
                _registered_method=True)
        self.Assets = channel.unary_unary(
                '/grpc.tradeapi.v1.assets.AssetsService/Assets',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.AssetsRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.AssetsResponse.FromString,
                _registered_method=True)
        self.GetAsset = channel.unary_unary(
                '/grpc.tradeapi.v1.assets.AssetsService/GetAsset',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetResponse.FromString,
                _registered_method=True)
        self.GetAssetParams = channel.unary_unary(
                '/grpc.tradeapi.v1.assets.AssetsService/GetAssetParams',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetParamsRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetParamsResponse.FromString,
                _registered_method=True)
        self.OptionsChain = channel.unary_unary(
                '/grpc.tradeapi.v1.assets.AssetsService/OptionsChain',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.OptionsChainRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.OptionsChainResponse.FromString,
                _registered_method=True)
        self.Schedule = channel.unary_unary(
                '/grpc.tradeapi.v1.assets.AssetsService/Schedule',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ScheduleRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ScheduleResponse.FromString,
                _registered_method=True)
        self.Clock = channel.unary_unary(
                '/grpc.tradeapi.v1.assets.AssetsService/Clock',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ClockRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ClockResponse.FromString,
                _registered_method=True)


class AssetsServiceServicer(object):
    def Exchanges(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Assets(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAsset(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAssetParams(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OptionsChain(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Schedule(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Clock(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AssetsServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Exchanges': grpc.unary_unary_rpc_method_handler(
                    servicer.Exchanges,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ExchangesRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ExchangesResponse.SerializeToString,
            ),
            'Assets': grpc.unary_unary_rpc_method_handler(
                    servicer.Assets,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.AssetsRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.AssetsResponse.SerializeToString,
            ),
            'GetAsset': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAsset,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetResponse.SerializeToString,
            ),
            'GetAssetParams': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAssetParams,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetParamsRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetParamsResponse.SerializeToString,
            ),
            'OptionsChain': grpc.unary_unary_rpc_method_handler(
                    servicer.OptionsChain,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.OptionsChainRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.OptionsChainResponse.SerializeToString,
            ),
            'Schedule': grpc.unary_unary_rpc_method_handler(
                    servicer.Schedule,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ScheduleRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ScheduleResponse.SerializeToString,
            ),
            'Clock': grpc.unary_unary_rpc_method_handler(
                    servicer.Clock,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ClockRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ClockResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'grpc.tradeapi.v1.assets.AssetsService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('grpc.tradeapi.v1.assets.AssetsService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class AssetsService(object):
    @staticmethod
    def Exchanges(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.assets.AssetsService/Exchanges',
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ExchangesRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ExchangesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Assets(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.assets.AssetsService/Assets',
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.AssetsRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.AssetsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetAsset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.assets.AssetsService/GetAsset',
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetAssetParams(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.assets.AssetsService/GetAssetParams',
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetParamsRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.GetAssetParamsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def OptionsChain(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.assets.AssetsService/OptionsChain',
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.OptionsChainRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.OptionsChainResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Schedule(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.assets.AssetsService/Schedule',
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ScheduleRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ScheduleResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Clock(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.assets.AssetsService/Clock',
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ClockRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_assets_dot_assets__service__pb2.ClockResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import abc
from abc import abstractmethod
from typing import Any

from ziplime.data.domain.data_bundle import DataBundle
from ziplime.data.services.bundle_storage import BundleStorage


class BundleRegistry(abc.ABC):
    """
    The BundleRegistry class serves as a registry to manage data bundles.

    This class provides functionality to list, load, delete, and register
    data bundles, along with persisting and retrieving their metadata. It acts
    as an interface for various bundle storage systems, enabling management
    of metadata operations. Derived classes must implement the abstract
    methods to handle specific persistence and retrieval of bundle metadata.

    """

    @abstractmethod
    async def list_bundles(self) -> list[dict[str, Any]]:
        """
        Method for listing bundles where each bundle
        is represented as a dictionary containing multiple key-value pairs.

        Returns:
            list[dict[str, Any]]: A list of dictionaries, each representing a bundle with
            specific attributes and details.
        """
        ...

    @abstractmethod
    async def list_bundles_by_name(self, bundle_name: str) -> list[dict[str, Any]]:
        """
        Retrieves a list of bundles filtered by their name.

        Args:
            bundle_name: The name of the bundle used to filter the results.

        Returns:
            A list of dictionaries, where each dictionary contains details of a bundle
            matching the specified name.
        """
        ...

    @abstractmethod
    async def load_bundle_metadata(self, bundle_name: str, bundle_version: str | None) -> dict[str, Any] | None:
        """
        Asynchronously loads metadata for a specified bundle.

        Attempts to retrieve metadata for a bundle identified by its name and optional
        version. Returns None if the metadata cannot be found.

        Args:
            bundle_name: The name of the bundle whose metadata is being loaded.
            bundle_version: The optional version of the bundle. If None, the latest
                version metadata will be retrieved.

        Returns:
            A dictionary containing the metadata of the bundle if available, or None if
            the metadata cannot be retrieved.
        """
        ...

    @abstractmethod
    async def delete_bundle(self):
        """
        Method for deleting a bundle.

        Raises:
            ValueError: If bundle cannot be found
        """
        ...

    @abstractmethod
    async def persist_metadata(self, data_bundle: DataBundle, metadata: dict[str, Any]):
        """
        Persists metadata associated with a data bundle.

        Args:
            data_bundle: The DataBundle object representing the data whose metadata is to
                be persisted.
            metadata: A dictionary containing metadata as key-value pairs. The keys
                represent metadata fields and corresponding values provide the field data.

        Returns:
            None. This method is intended for storing metadata and does not return a value.
        """
        ...

    @abstractmethod
    async def get_bundle_metadata(self, data_bundle: DataBundle, bundle_storage: BundleStorage) -> dict[str, Any]:
        """
        Method for retrieving metadata about a data bundle.

        Args:
            data_bundle (DataBundle): The data bundle object for which metadata needs
                to be fetched.
            bundle_storage (BundleStorage): The storage mechanism handling the bundle,
                used to retrieve associated data or configurations.

        Returns:
            dict[str, Any]: A dictionary containing metadata about the data bundle.
        """
        ...

    async def register_bundle(self, data_bundle: DataBundle, bundle_storage: BundleStorage):
        """
        Registers a data bundle and persists its associated metadata.

        The method processes a given data bundle, retrieves its metadata, and ensures the
        metadata is saved for future reference.

        Args:
            data_bundle (DataBundle): The data bundle to register.
            bundle_storage (BundleStorage): Storage reference related to the data bundle.

        Returns:
            None
        """
        metadata = await self.get_bundle_metadata(data_bundle=data_bundle, bundle_storage=bundle_storage)
        await self.persist_metadata(data_bundle=data_bundle, metadata=metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1.marketdata import marketdata_service_pb2 as grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2

GRPC_GENERATED_VERSION = '1.74.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in grpc/tradeapi/v1/marketdata/marketdata_service_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class MarketDataServiceStub(object):
    def __init__(self, channel):
        self.Bars = channel.unary_unary(
                '/grpc.tradeapi.v1.marketdata.MarketDataService/Bars',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.BarsRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.BarsResponse.FromString,
                _registered_method=True)
        self.LastQuote = channel.unary_unary(
                '/grpc.tradeapi.v1.marketdata.MarketDataService/LastQuote',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.QuoteRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.QuoteResponse.FromString,
                _registered_method=True)
        self.OrderBook = channel.unary_unary(
                '/grpc.tradeapi.v1.marketdata.MarketDataService/OrderBook',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.OrderBookRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.OrderBookResponse.FromString,
                _registered_method=True)
        self.LatestTrades = channel.unary_unary(
                '/grpc.tradeapi.v1.marketdata.MarketDataService/LatestTrades',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.LatestTradesRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.LatestTradesResponse.FromString,
                _registered_method=True)
        self.SubscribeQuote = channel.unary_stream(
                '/grpc.tradeapi.v1.marketdata.MarketDataService/SubscribeQuote',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeQuoteRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeQuoteResponse.FromString,
                _registered_method=True)
        self.SubscribeOrderBook = channel.unary_stream(
                '/grpc.tradeapi.v1.marketdata.MarketDataService/SubscribeOrderBook',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeOrderBookRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeOrderBookResponse.FromString,
                _registered_method=True)
        self.SubscribeLatestTrades = channel.unary_stream(
                '/grpc.tradeapi.v1.marketdata.MarketDataService/SubscribeLatestTrades',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeLatestTradesRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeLatestTradesResponse.FromString,
                _registered_method=True)
        self.SubscribeBars = channel.unary_stream(
                '/grpc.tradeapi.v1.marketdata.MarketDataService/SubscribeBars',
                request_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeBarsRequest.SerializeToString,
                response_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeBarsResponse.FromString,
                _registered_method=True)


class MarketDataServiceServicer(object):
    def Bars(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LastQuote(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OrderBook(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LatestTrades(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeQuote(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeOrderBook(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeLatestTrades(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeBars(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MarketDataServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Bars': grpc.unary_unary_rpc_method_handler(
                    servicer.Bars,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.BarsRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.BarsResponse.SerializeToString,
            ),
            'LastQuote': grpc.unary_unary_rpc_method_handler(
                    servicer.LastQuote,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.QuoteRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.QuoteResponse.SerializeToString,
            ),
            'OrderBook': grpc.unary_unary_rpc_method_handler(
                    servicer.OrderBook,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.OrderBookRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.OrderBookResponse.SerializeToString,
            ),
            'LatestTrades': grpc.unary_unary_rpc_method_handler(
                    servicer.LatestTrades,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.LatestTradesRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.LatestTradesResponse.SerializeToString,
            ),
            'SubscribeQuote': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeQuote,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeQuoteRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeQuoteResponse.SerializeToString,
            ),
            'SubscribeOrderBook': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeOrderBook,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeOrderBookRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeOrderBookResponse.SerializeToString,
            ),
            'SubscribeLatestTrades': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeLatestTrades,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeLatestTradesRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeLatestTradesResponse.SerializeToString,
            ),
            'SubscribeBars': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeBars,
                    request_deserializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeBarsRequest.FromString,
                    response_serializer=grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeBarsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'grpc.tradeapi.v1.marketdata.MarketDataService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('grpc.tradeapi.v1.marketdata.MarketDataService', rpc_method_handlers)


class MarketDataService(object):
    @staticmethod
    def Bars(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.marketdata.MarketDataService/Bars',
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.BarsRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.BarsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def LastQuote(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.marketdata.MarketDataService/LastQuote',
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.QuoteRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.QuoteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def OrderBook(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.marketdata.MarketDataService/OrderBook',
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.OrderBookRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.OrderBookResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def LatestTrades(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/grpc.tradeapi.v1.marketdata.MarketDataService/LatestTrades',
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.LatestTradesRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.LatestTradesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SubscribeQuote(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/grpc.tradeapi.v1.marketdata.MarketDataService/SubscribeQuote',
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeQuoteRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeQuoteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SubscribeOrderBook(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/grpc.tradeapi.v1.marketdata.MarketDataService/SubscribeOrderBook',
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeOrderBookRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeOrderBookResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SubscribeLatestTrades(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/grpc.tradeapi.v1.marketdata.MarketDataService/SubscribeLatestTrades',
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeLatestTradesRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeLatestTradesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SubscribeBars(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/grpc.tradeapi.v1.marketdata.MarketDataService/SubscribeBars',
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeBarsRequest.SerializeToString,
            grpc_dot_tradeapi_dot_v1_dot_marketdata_dot_marketdata__service__pb2.SubscribeBarsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import datetime
import multiprocessing
import os
from typing import Self

import aiocache
import grpc
import structlog

import polars as pl
from aiocache import Cache

from ziplime.assets.entities.asset import Asset
from ziplime.assets.entities.equity import Equity
from ziplime.assets.entities.equity_symbol_mapping import EquitySymbolMapping
from ziplime.assets.models.exchange_info import ExchangeInfo
from ziplime.data.data_sources.asset_data_source import AssetDataSource
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1.auth import auth_service_pb2_grpc, auth_service_pb2
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1.assets import assets_service_pb2_grpc, \
    assets_service_pb2


class GrpcAssetDataSource(AssetDataSource):
    def __init__(self, authorization_token: str, server_url: str,
                 maximum_threads: int | None = None):
        super().__init__()
        self._logger = structlog.get_logger(__name__)
        self._server_url = server_url
        self._authorization_token = authorization_token
        if maximum_threads is not None:
            self._maximum_threads = min(multiprocessing.cpu_count() * 2, maximum_threads)
        else:
            self._maximum_threads = multiprocessing.cpu_count() * 2

    @aiocache.cached(cache=Cache.MEMORY)
    async def get_token(self) -> str:
        credentials = grpc.ssl_channel_credentials()
        async with grpc.aio.secure_channel(self._server_url, credentials) as channel:
            stub = auth_service_pb2_grpc.AuthServiceStub(channel)
            auth_request = auth_service_pb2.AuthRequest()
            auth_request.secret = self._authorization_token
            response = await stub.Auth(auth_request)
        return response.token

    async def get_assets(self, **kwargs) -> list[Asset]:
        request = assets_service_pb2.AssetsRequest()
        token = await self.get_token()
        metadata = [('authorization', token)]
        credentials = grpc.ssl_channel_credentials()

        async with grpc.aio.secure_channel(self._server_url, credentials) as channel:
            stub = assets_service_pb2_grpc.AssetsServiceStub(channel)
            response_stream = await stub.Assets(request, metadata=metadata)

            self._logger.info(f"Got {len(response_stream.assets)} assets from GRPC asset data source.")

        asset_start_date = datetime.datetime(year=1900, month=1, day=1, tzinfo=datetime.timezone.utc)
        asset_end_date = datetime.datetime(year=2099, month=1, day=1, tzinfo=datetime.timezone.utc)

        equities = {}
        for asset in response_stream.assets:
            if asset.ticker not in equities:
                equities[asset.ticker] =  Equity(
                        asset_name=asset.symbol,
                            symbol_mapping={
                                asset.mic: EquitySymbolMapping(
                                    symbol=asset.ticker,
                                    exchange_name=asset.mic,
                                    start_date=asset_start_date,
                                    end_date=asset_end_date,
                                    company_symbol="",
                                    share_class_symbol=""
                                )
                                # "LIME": EquitySymbolMapping(
                                #     symbol=asset.symbol,
                                #     exchange_name="LIME",
                                #     start_date=asset_start_date,
                                #     end_date=asset_end_date,
                                #     company_symbol="",
                                #     share_class_symbol=""
                                # )
                            },
                            sid=None,
                            start_date=asset_start_date,
                            end_date=asset_end_date,
                            auto_close_date=asset_end_date,
                            first_traded=asset_start_date,
                            mic=asset.mic
                        )
            else:
                equities[asset.ticker].symbol_mapping[asset.mic] =EquitySymbolMapping(
                                    symbol=asset.ticker,
                                    exchange_name=asset.mic,
                                    start_date=asset_start_date,
                                    end_date=asset_end_date,
                                    company_symbol="",
                                    share_class_symbol=""
                                )

        return list(equities.values())

    async def get_exchanges(self, **kwargs) ->list[ExchangeInfo]:
        request = assets_service_pb2.ExchangesRequest()
        token = await self.get_token()
        metadata = [('authorization', token)]
        credentials = grpc.ssl_channel_credentials()

        async with grpc.aio.secure_channel(self._server_url, credentials) as channel:
            stub = assets_service_pb2_grpc.AssetsServiceStub(channel)
            response_stream = await stub.Exchanges(request, metadata=metadata)
            exchanges = [ExchangeInfo(exchange=exchange.mic, canonical_name=exchange.name, country_code="US")
                         for exchange in response_stream.exchanges]
            self._logger.info(f"Got {len(exchanges)} exchanges from GRPC asset data source.")
        return exchanges

    async def get_constituents(self, index: str) -> pl.DataFrame:
        assets = self._limex_client.constituents(index)
        return assets

    @classmethod
    def from_env(cls) -> Self:
        token = os.environ.get("GRPC_TOKEN", None)
        server_url = os.environ.get("GRPC_SERVER_URL")
        maximum_threads = os.environ.get("GRPC_MAXIMUM_THREADS", None)
        if token is None:
            raise ValueError("Missing GRPC_TOKEN environment variable.")
        return cls(server_url=server_url, authorization_token=token, maximum_threads=maximum_threads)



import asyncio
import datetime
import multiprocessing
import os
import sys
import time
from typing import Self

import aiocache
import grpc
import structlog
from aiocache import Cache
from asyncclick import progressbar
from google.type.date_pb2 import Date

import polars as pl

from ziplime.data.services.data_bundle_source import DataBundleSource
# from ziplime.data.data_sources.grpc.grpc_stubs.grpc.ta import candles_pb2
# from ziplime.data.data_sources.grpc.grpc_stubs.grpc.ta import candles_pb2_grpc
# from ziplime.data.data_sources.grpc.grpc_stubs.proto.ta import ta_pb2
# from ziplime.data.data_sources.grpc.grpc_stubs.proto.common import securityidentifier_pb2
#
from google.protobuf.timestamp_pb2 import Timestamp
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1.auth import auth_service_pb2_grpc, auth_service_pb2
from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1.marketdata import marketdata_service_pb2_grpc, \
    marketdata_service_pb2
# from ziplime.data.data_sources.grpc.grpc_stubs.grpc.tradeapi.v1 import interval_pb2
from google.type import interval_pb2


class GrpcDataSource(DataBundleSource):
    def __init__(self, authorization_token: str, server_url: str,
                 maximum_threads: int | None = None):
        super().__init__()
        self._logger = structlog.get_logger(__name__)
        self._server_url = server_url
        self._authorization_token = authorization_token
        if maximum_threads is not None:
            self._maximum_threads = min(multiprocessing.cpu_count() * 2, maximum_threads)
        else:
            self._maximum_threads = multiprocessing.cpu_count() * 2

    @aiocache.cached(cache=Cache.MEMORY)
    async def get_token(self) -> str:
        credentials = grpc.ssl_channel_credentials()
        async with grpc.aio.secure_channel(self._server_url, credentials) as channel:
            stub = auth_service_pb2_grpc.AuthServiceStub(channel)
            auth_request = auth_service_pb2.AuthRequest()
            auth_request.secret = self._authorization_token
            response = await stub.Auth(auth_request)
        return response.token

    def _get_timeframe(self, frequency: datetime.timedelta) -> str:
        if frequency <= datetime.timedelta(minutes=1):
            return "TIME_FRAME_M1"
        elif frequency <= datetime.timedelta(minutes=5):
            return "TIME_FRAME_M5"
        elif frequency <= datetime.timedelta(minutes=15):
            return "TIME_FRAME_M15"
        elif frequency <= datetime.timedelta(minutes=30):
            return "TIME_FRAME_M30"
        elif frequency <= datetime.timedelta(hours=1):
            return "TIME_FRAME_H1"
        elif frequency <= datetime.timedelta(hours=2):
            return "TIME_FRAME_H2"
        elif frequency <= datetime.timedelta(hours=4):
            return "TIME_FRAME_H4"
        elif frequency <= datetime.timedelta(hours=8):
            return "TIME_FRAME_H8"
        elif frequency <= datetime.timedelta(days=1):
            return "TIME_FRAME_D"
        elif frequency <= datetime.timedelta(weeks=1):
            return "TIME_FRAME_W"
        elif frequency <= datetime.timedelta(days=31):
            return "TIME_FRAME_MN"
        elif frequency <= datetime.timedelta(days=95):
            return "TIME_FRAME_QR"

        raise ValueError(f"Unsupported frequency for Yahoo Finance {frequency}")

    async def fetch_historical_lime_trader_data(self,
                                                channel: grpc.aio.Channel,
                                                date_from: datetime.datetime,
                                                date_to: datetime.datetime,
                                                symbol: str,
                                                frequency: datetime.timedelta,
                                                ) -> tuple[pl.DataFrame, float, float]:
        duration_start = time.time()
        token = await self.get_token()

        stub = marketdata_service_pb2_grpc.MarketDataServiceStub(channel)
        metadata = [('authorization', token)]
        if "@" in symbol:
            ticker, mic = symbol.split("@")
        else:
            ticker, mic = symbol, "XNGS"
        timestamp_from = Timestamp()
        timestamp_to = Timestamp()
        timestamp_from.FromDatetime(date_from)
        timestamp_to.FromDatetime(date_to)
        interval = interval_pb2.Interval(start_time=timestamp_from, end_time=timestamp_to)

        bars_request = marketdata_service_pb2.BarsRequest(
            timeframe=self._get_timeframe(frequency=frequency),
            interval=interval)
        bars_request.symbol = f"{ticker}@{mic}"
        self._logger.info(f"Fetching market data for symbol {symbol} - {date_from} to {date_to}",
                          symbol=symbol, start_date=date_from, end_date=date_to)
        total_requests_time = 0
        request_start = time.time()
        response = await stub.Bars(bars_request, metadata=metadata)
        duration = time.time() - request_start
        total_requests_time += duration

        rows = [
            {
                "date": datetime.datetime.fromtimestamp(candle.timestamp.seconds,
                                          tz=date_from.tzinfo),
                "open": float(candle.open.value),
                "high": float(candle.high.value),
                "low": float(candle.low.value),
                "close": float(candle.close.value),
                "volume": int(float(candle.volume.value)),
                "exchange": mic,
                "exchange_country": "US",
                "price": float(candle.close.value),
                "symbol": ticker,
            } for candle in response.bars
        ]
        self._logger.info(
            f"Fetched market data for symbol {symbol} - {date_from} to {date_to} in {duration:.2f}",
            symbol=symbol, start_date=date_from, end_date=date_to, duration=duration)

        try:

            df = pl.DataFrame(rows, schema=[("open", pl.Float64()), ("close", pl.Float64()),
                                            ("price", pl.Float64()),
                                            ("high", pl.Float64()), ("low", pl.Float64()),
                                            ("volume", pl.Float64()),
                                            ("date", pl.Datetime(time_zone=date_from.tzinfo)), ("exchange", pl.String),
                                            ("exchange_country", pl.String), ("symbol", pl.String)
                                            ])
            duration_total = time.time() - duration_start

            self._logger.info(
                f"Retrieved {len(rows)} candles for {symbol} in {duration_total:.2f}s. "
                f"Total requests time: {total_requests_time:.2f}s",
                total_duration=duration_total, requests_duration=total_requests_time
            )

            return df, duration_total, total_requests_time
        except grpc.RpcError as e:
            self._logger.exception(f"Failed to get day candles for {symbol}")
            raise

    async def get_data(self, symbols: list[str],
                       frequency: datetime.timedelta,
                       date_from: datetime.datetime,
                       date_to: datetime.datetime,
                       **kwargs
                       ) -> pl.DataFrame:
        async def fetch_historical(symbol: str, start_date: datetime.datetime,
                                   end_date: datetime.datetime) -> tuple[pl.DataFrame | None, float, float]:
            try:
                credentials = grpc.ssl_channel_credentials()
                async with grpc.aio.secure_channel(self._server_url, credentials) as channel:

                    result, duration, requests_time = await self.fetch_historical_lime_trader_data(channel=channel,
                                                                                                   date_from=start_date,
                                                                                                   date_to=end_date,
                                                                                                   symbol=symbol,
                                                                                                   frequency=frequency)
                    return result, duration, requests_time
            except Exception as e:
                self._logger.exception(
                    f"Exception fetching historical data for symbol {symbol}, date_from={start_date}, date_to={end_date}. Skipping."
                )
                return None, 0, 0

        total_days = (date_to - date_from).days
        final = pl.DataFrame()

        with progressbar(length=len(symbols) * total_days, label="Downloading historical data from Lime Trader",
                         file=sys.stdout) as pbar:

            if frequency >= datetime.timedelta(days=1):
                maximum_batch = datetime.timedelta(days=7200)
            elif frequency >= datetime.timedelta(hours=1):
                maximum_batch = datetime.timedelta(days=365)
            elif frequency >= datetime.timedelta(minutes=1):
                maximum_batch = datetime.timedelta(days=180)
            elif frequency >= datetime.timedelta(seconds=1):
                maximum_batch = datetime.timedelta(days=30)

            tasks = []
            batch_start_date = date_from
            while batch_start_date < date_to:

                batch_end_date = batch_start_date + maximum_batch
                if batch_end_date > date_to:
                    batch_end_date = date_to

                tasks.extend(
                    fetch_historical(
                        symbol=symbol, start_date=batch_start_date, end_date=batch_end_date
                    ) for symbol in symbols
                )
                batch_start_date = batch_end_date

            total_duration = 0
            total_requests_duration = 0
            res = await asyncio.gather(*tasks)

            for item in res:
                df, duration, requests_duration = item
                total_duration += duration
                total_requests_duration += requests_duration
                pbar.update(total_days)
                if df is None:
                    continue
                final = pl.concat([final, df])
            self._logger.info(
                f"Retrieved {len(final)} candles for {symbols} in {total_duration:.2f}s. "
                f"Total requests time: {total_requests_duration:.2f}s",
                total_duration=total_duration, requests_duration=total_requests_duration
            )

        return final

    @classmethod
    def from_env(cls) -> Self:
        token = os.environ.get("GRPC_TOKEN", None)
        server_url = os.environ.get("GRPC_SERVER_URL")
        maximum_threads = os.environ.get("GRPC_MAXIMUM_THREADS", None)
        if token is None:
            raise ValueError("Missing GRPC_TOKEN environment variable.")
        return cls(server_url=server_url, authorization_token=token, maximum_threads=maximum_threads)
